/*----------------------------------------------------------------------
copyright (c) 2008 Georget Fabien

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--------------------------------------------------------------------------
File : ExploTab.cpp
Program : gt2n
Date : 16/06/08
--------------------------------------------------------------------------
Nature : C++ file for the explorator widget
--------------------------------------------------------------------------*/


#include <QStandardItemModel>
#include <QTreeView>
//#include <QListView>
#include <QAbstractItemView>
#include <QGridLayout>
#include <QPushButton>
#include <QWidget>
#include <QDirModel>
#include <QAction>
#include <QFileDialog>
#include <QContextMenuEvent>
#include <QMenu>
#include <QDockWidget>
//#include <QModelIndex>

#include "ExploTab.h"

using namespace std;
ExploTab::ExploTab()
{
}

ExploTab::ExploTab(QDockWidget *dock,MainWindow *mainWind):QWidget(dock)
{
  fenPrincipale = mainWind;
    createAction();
  model = new  QStandardItemModel(0,3);
  model->setHeaderData(0, Qt::Horizontal, "name");
  model->setHeaderData(1,Qt::Horizontal, "size");
  model->setHeaderData(2, Qt::Horizontal, "Type");

  fileList = new QTreeView(this);
  fileList->setModel(model);
  /*
  modelDir = new QDirModel;
  compView = new QListView;
  compView->setViewMode(QListView::IconMode);
  compView->setModel(modelDir);
  //compView->setDragDropMode(QAbstractItemView::DragDrop); 
  compView->setRootIndex(modelDir->index(QDir::homePath()+"/nxt"));
 */

  QPushButton *button = new QPushButton("Refresh");
  connect(button,SIGNAL(clicked()),this,SLOT(s_listFile()));

  QGridLayout *gridLayout = new QGridLayout;
  gridLayout->addWidget(fileList,0,0);
  //gridLayout->addWidget(compView,0,1);
  gridLayout->addWidget(button);
  this->setLayout(gridLayout);

  connect(fileList,SIGNAL(clicked(const QModelIndex)),this,SLOT(s_affFile(const QModelIndex)));
}


/* ------------------------------------------------------------------------\
|			       PUBLIC                                      |
\-------------------------------------------------------------------------*/

// add a file on the model
void ExploTab::addFile(string type,string name, int size)
{
  QString str;
  model->insertRow(0);
  model->setData(model->index(0,0),str.fromStdString(name));
  model->setData(model->index(0,1),size);
  model->setData(model->index(0,2),str.fromStdString(type));
}


// List NXT files 
// call methods of fenPrincipale to get file's descriptions
void ExploTab::listFile()
{
    clean();
    fileDesc = fenPrincipale->getFirstFile();
    while (fileDesc.status==0)
    {
      addFile(fileDesc.type,fileDesc.filename,fileDesc.filesize);
      fileDesc = fenPrincipale->getNextFile(fileDesc.cmdhdl);
    }
    fenPrincipale->closeSearchFile(fileDesc.cmdhdl);
}

// return the name of the actived file
QString ExploTab::fileNameActived()
{
  return model->item(fileList->currentIndex().row(),0)->text();
}

// clean the model of the listView
void ExploTab::clean()
{
  delete model;
  model = new QStandardItemModel(0,3);
  model->setHeaderData(0, Qt::Horizontal, "name");
  model->setHeaderData(1, Qt::Horizontal, "size");
  model->setHeaderData(2,Qt::Horizontal, "type");
  fileList->setModel(model);
}

// method to enable/disable action
void ExploTab::actEnableConnect(bool boolean)
{
  uploadAct->setEnabled(boolean);
  downloadAct->setEnabled(boolean);
  refreshAct->setEnabled(boolean);
  deleteAct->setEnabled(boolean);
}

/* ------------------------------------------------------------------------\
|			       PROTECTED                                   |
\-------------------------------------------------------------------------*/


// to display the menu (right click !)
void ExploTab::contextMenuEvent(QContextMenuEvent *event)
{
  QMenu menu(this);
  menu.addAction(refreshAct);
  menu.addSeparator();
  menu.addAction(uploadAct);
  menu.addAction(downloadAct);
  menu.addAction(deleteAct);
  menu.exec(event->globalPos());
}

/* ------------------------------------------------------------------------\
|			       SLOT                                        |
\-------------------------------------------------------------------------*/

// a SLOT to list files
void ExploTab::s_listFile()
{
  if(fenPrincipale->brickIsHere())
  {
    clean();
    listFile();
  }
}

// a SLOT to delete the actived file
void ExploTab::s_delFile() //SLOT
{
  if(fenPrincipale->brickIsHere())
  {
    fenPrincipale->deleteFile(fileNameActived());
    listFile();
  }
}


// a SLOT to display on the statusBar the name of the actived file
void ExploTab::s_affFile(const QModelIndex index)
{
    
    fenPrincipale->afficherMessage(model->item(index.row(),0)->text());
}

// a SLOT to download file from the computer
void ExploTab::s_downFile() //SLOT
{
  if (fenPrincipale->brickIsHere())
  {
    QString dossier = QFileDialog::getExistingDirectory(this);
    if (!dossier.isEmpty())
    {
      fenPrincipale->afficherMessage(dossier+fileNameActived(),2000);
      fenPrincipale->downloadFile(dossier+fileNameActived());
    }
    else
    {
      fenPrincipale->afficherMessage(tr("action cancelled"));
    } 
  }
}


/* ------------------------------------------------------------------------\
|			       PRIVATE                                     |
\-------------------------------------------------------------------------*/

// create actions
void ExploTab::createAction()
{
  // Connect a file
  uploadAct = new QAction(tr("Upload"),this);
    uploadAct->setStatusTip(tr("Upload a file on the brick"));
    uploadAct->setEnabled(false);
    connect(uploadAct,SIGNAL(triggered()),fenPrincipale,SLOT(s_uploadFile()));

  // Download a file
  downloadAct = new QAction(tr("Download"),this);
    downloadAct->setStatusTip(tr("Download a file from the brick"));
    downloadAct->setEnabled(false);
    connect(downloadAct,SIGNAL(triggered()),this,SLOT(s_downFile()));
    
  // Delete a file 
  deleteAct = new QAction(tr("Delete"),this);
    deleteAct->setStatusTip(tr("Delete a file from the brick"));
    deleteAct->setEnabled(false);
    connect(deleteAct,SIGNAL(triggered()),this,SLOT(s_delFile()));

 // Refresh the list
  refreshAct = new QAction(tr("Refresh"),this);
    refreshAct->setStatusTip(tr("Refresh the list of files"));
    refreshAct->setEnabled(false);
    connect(refreshAct,SIGNAL(triggered()),this,SLOT(s_listFile()));
}


